using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using OpenTK.Mathematics;

namespace Luski.GUI.MainScreen.UI.LuskiControls;

public class MessageCompressedFlow : UserControl
{
    public MessageCompressedFlow()
    {
        Controls.ControlAfterAdded += ControlsOnControlAdded;
        Controls.ControlRemoved += ControlsOnControlRemoved;
        SizeChanged += OnSizeChanged;
        WallSpacing = new(5.ScaleInt());
    }
    
    public Vector4i WallSpacing { get; set; }

    public event Func<RenderObjectCore, Task>? SizeUpdateNotIgnored; 

    private bool IgnoreNextSizeChange;

    private Task OnSizeChanged(RenderObjectCore arg)
    {
        if (IgnoreNextSizeChange)
        {
            IgnoreNextSizeChange = false;
            return Task.CompletedTask;
        }
        UpdateControlesFromIndex(0);
        return Task.CompletedTask;
    }

    private Task ControlsOnControlRemoved()
    {
        UpdateControlesFromIndex(0);
        return Task.CompletedTask;
    }
    
    private Task ControlsOnControlAdded(int arg1, RenderObjectCore arg2)
    {
        if (arg1 != 0)
        {
            arg2.Margins = new(0, 3.ScaleInt(), 0, 0);
            arg2.Location = new(WallSpacing.X + arg2.Margins.X, Controls[arg1-1].Location.X + Controls[arg1-1].Size.Y + Controls[arg1-1].Margins.W + arg2.Margins.Y);
        }
        else
        {
            arg2.Location = new(WallSpacing.X + arg2.Margins.X, WallSpacing.Y + arg2.Margins.Y);
        }

        if (arg2 is CompressedFlow c)
        {
            c.Anchor = ObjectAnchor.Left | ObjectAnchor.Top | ObjectAnchor.Right;
            c.SizeUpdateNotIgnored += o =>
            {
                UpdateControlesFromIndex(arg1 + 1);
                return Task.CompletedTask;
            };
        }
        UpdateControlesFromIndex(arg1);
        return Task.CompletedTask;
    }

    private void UpdateControlesFromIndex(int index)
    {
        if (Controls.Length == 0)
        {
            IgnoreNextSizeChange = true;
            Size = new(Size.X, 0);
            if (SizeUpdateNotIgnored is not null) _ = SizeUpdateNotIgnored.Invoke(this);
            return;
        }
        if (index == 0) index++;
        for (int i = index; i < Controls.Length; i++)
        {
            Controls[i].Location = new(WallSpacing.X + Controls[i].Margins.X,
                Controls[i - 1].Location.Y + Controls[i - 1].Size.Y + Controls[i - 1].Margins.W + Controls[i].Margins.Y);
        }

        if (Controls[Controls.Length - 1].Location.Y + Controls[Controls.Length - 1].Size.Y + Controls[Controls.Length - 1].Margins.Z + WallSpacing.W != Size.Y)
        {
            IgnoreNextSizeChange = true;
            Size = new(Size.X,
                Controls[Controls.Length - 1].Location.Y + Controls[Controls.Length - 1].Size.Y + Controls[Controls.Length - 1].Margins.Z + WallSpacing.W);
            if (SizeUpdateNotIgnored is not null) _ = SizeUpdateNotIgnored.Invoke(this);
        }
    }
}