using System.Numerics;
using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using Luski.Enums;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common;
using OpenTK.Windowing.GraphicsLibraryFramework;

namespace Luski.GUI.MainScreen.UI.LuskiControls;

public class NumberSelector<TNumber> : UserControl where TNumber : INumber<TNumber>
{
    public event Func<NumberSelector<TNumber>, Task>? ValueChanged; 
    
    public int Space
    {
        get
        {
            return space;
        }
        set
        {
            space = value;
            progressBar.Location = new(value, progressBar.Location.Y);
            progressBar.Size = new(base.Size.X - value - value, progressBar.Size.Y);
            //progressBar.SetLocation(value, progressBar.Location.Y);
            //progressBar.SetSize(base.Size.X - value - value, progressBar.Size.Y);
            progressBar.UpdateProgress();
            Min = Min;
        }
    }
    
    public TNumber Min
    {
        get
        {
            return _Min;
        }
        set
        {
            _Min = value;
            if (Percent) L1.Text = (value * TNumber.CreateChecked(100))+ "%" + Suffix;
            else L1.Text = value + Suffix;
            int x = GetX(value);
            L1.SetLocation(x - (L1.Size.X/2) + space, L1.Location.Y);
            L1.ForceDistanceUpdate(this);
            MinLine.SetLocation(x + space, MinLine.Location.Y);
            x = GetX(_Default);
            L2Default.SetLocation(x - (L2Default.Size.X/2) + space, L2Default.Location.Y);
            L2Default.ForceDistanceUpdate(this);
            Max = Max;
            Value = Value;
        }
    }

    public TNumber Max
    {
        get
        {
            return progressBar.MaxProgressValue + Min;
        }
        set
        {
            progressBar.MaxProgressValue = value - Min;
            if (Percent) L3.Text = (value * TNumber.CreateChecked(100))+ "%" + Suffix;
            else L3.Text = value + Suffix;
            int x = GetX(value);
            L3.SetLocation(x - (L3.Size.X/2) + space, L3.Location.Y);
            L3.ForceDistanceUpdate(this);
            MaxLine.SetLocation(x + space, MaxLine.Location.Y);
            MaxLine.ForceDistanceUpdate(this);
            x = GetX(_Default);
            L2Default.SetLocation(x - (L2Default.Size.X/2) + space, L2Default.Location.Y);
            L2Default.ForceDistanceUpdate(this);
            DefaltLine.SetLocation(x - (DefaltLine.Size.X/2) + space, DefaltLine.Location.Y);
            DefaltLine.ForceDistanceUpdate(this);
            progressBar.UpdateProgress();
        }
    }

    private int GetX(TNumber number)
    {
        if (number >= Max) return progressBar.GetParentLocation(Max - Min, IgnoreEnd: true);
        if (number <= Min) return progressBar.GetParentLocation(TNumber.Zero, IgnoreEnd: true);
        return progressBar.GetParentLocation(number - Min, IgnoreEnd: true);
    }

    private TNumber Extra = TNumber.Zero;

    public TNumber Default
    {
        get => _Default;
        set
        {
            if (_Default == value) return;
            _Default = value;
            if (Percent) L2Default.Text = (value * TNumber.CreateChecked(100))+ "%" + Suffix;
            else L2Default.Text = value + Suffix;
            int x = GetX(value);
            L2Default.SetLocation(x - (L2Default.Size.X/2) + space, L2Default.Location.Y);
            DefaltLine.SetLocation(x - (DefaltLine.Size.X/2) + space, DefaltLine.Location.Y);
            DefaltLine.ForceDistanceUpdate(this);
            L2Default.ForceDistanceUpdate(this);
        }
    }
    public TNumber Value
    {
        get
        {
            return progressBar.ProgressValue + Min - Extra;
        }
        set
        {
            int i = 0;
            TNumber tmp = value - Min;
            AddBlockDraw();
            if (value < Min)
            {
                Extra = Min - value;
                progressBar.ProgressValue = TNumber.Zero;
            }
            else
            {
                Extra = TNumber.Zero;
                progressBar.ProgressValue = tmp;
            }
            
            Type tt = typeof(TNumber);


            if (Percent)
            {
                if (tt == typeof(double))
                    L2Cursor.Text = Math.Round((double)(object)(value * TNumber.CreateChecked(100)), 0) + "%" + Suffix;
                else if (tt == typeof(float))
                    L2Cursor.Text = Math.Round((float)(object)(value * TNumber.CreateChecked(100)), 0) + "%" + Suffix;
                else
                    L2Cursor.Text = (value * TNumber.CreateChecked(100)) + "%" + Suffix;
            }
            else
            {
                if (tt == typeof(double))
                    L2Cursor.Text = Math.Round((double)(object)value, 2) + Suffix;
                else if (tt == typeof(float))
                    L2Cursor.Text = Math.Round((float)(object)value, 2) + Suffix;
                else
                    L2Cursor.Text = value.ToString() + Suffix;
            }
            int x = GetX(value);
            if (value > Max || value < Min) L2Cursor.Color = Color4.Gold;
            else L2Cursor.Color = Color4.White;
            L2Cursor.SetLocation(x - (L2Cursor.Size.X/2) + space, L2Cursor.Location.Y);
            Cursor.SetLocation(x - (Cursor.Size.X/2) + space, MaxLine.Location.Y);
            Cursor.ForceDistanceUpdate(this);
            L3.Visible = L2Cursor.Location.X + L2Cursor.Size.X < L3.Location.X;
            L1.Visible = L2Cursor.Location.X > L1.Location.X + L1.Size.X;
            L2Cursor.Visible = value != Default;
            RemoveBlockDraw();
            progressBar.UpdateProgress();
            if (Loaded && BlockEvents == 0 && ValueChanged is not null) ValueChanged.Invoke(this);
        }
    }
    public bool LockToPoints { get; set; } = false;
    public string Suffix = "";
    
    
    private ProgressBar<TNumber> progressBar = new();
    private Rectangle Cursor = new(), MinLine = new(), MaxLine = new(), DefaltLine = new();
    private List<(TNumber, Rectangle)> ExtraPoints = new();
    private int space = 5;
    private TNumber _Min = TNumber.Zero, _Default = TNumber.Zero;
    public Label
        L1 = new(Globals.SliderFont),
        L2Default = new(Globals.SliderFont),
        L2Cursor = new(Globals.SliderFont),
        L3 = new(Globals.SliderFont);
    
    public bool Percent { get; set; }

    public override void KeyEventChain(KeyboardKeyEventArgs KeyArgs)
    {
        base.KeyEventChain(KeyArgs);
        if (Disabled) return;
        if (KeyArgs.Key == Keys.Left && (Value > Min || KeyArgs.Control)) Value--;
        else if (KeyArgs.Key == Keys.Right && (Value < Max || KeyArgs.Control)) Value++;
    }

    public NumberSelector()
    {
        base.SetSize(300.ScaleInt());
        progressBar.SetSize(base.Size.X, 8.ScaleInt());
        progressBar.Anchor = ObjectAnchor.Left | ObjectAnchor.Top | ObjectAnchor.Right;
        progressBar.DisallowAboveMax = false;
        progressBar.DrawingGap.X = 1.ScaleInt();
        progressBar.UpdateProgress();
        progressBar.DrawingGap.X = 0;
        progressBar.DrawingGap.Y = progressBar.DrawingGap.X;
        progressBar.DrawingGap.Z = progressBar.DrawingGap.X;
        progressBar.DrawingGap.W = progressBar.DrawingGap.X;
        progressBar.ProgressGap.X = 4.ScaleInt();
        progressBar.ProgressGap.Y = progressBar.ProgressGap.X;
        progressBar.BackgroundColor = BackColor;
        progressBar.ProgressColor = FillColor;
        progressBar.UpdateOnDraw = false;
        progressBar.Anchor = ObjectAnchor.Left | ObjectAnchor.Right;
        progressBar.UpdateProgress();
        progressBar.SetLocation(progressBar.Location.X, base.Size.Y-progressBar.Size.Y-progressBar.Size.Y);
        progressBar.Clicked += ProgressBarOnClicked;
        Cursor.SetSize(10.ScaleInt(),24.ScaleInt());
        Cursor.SetLocation(0, base.Size.Y-Cursor.Size.Y);
        Cursor.BackgroundColor = CursorColor;
        Cursor.Anchor = ObjectAnchor.PreventWidthChange;
        MinLine.SetSize(2.ScaleInt(), Cursor.Size.Y);
        MinLine.SetLocation(0, Cursor.Location.Y);
        MaxLine.SetSize(MinLine.Size.X, Cursor.Size.Y);
        MaxLine.SetLocation(0, Cursor.Location.Y);
        DefaltLine.SetSize(MinLine.Size.X, Cursor.Size.Y);
        DefaltLine.SetLocation(0, Cursor.Location.Y);
        L1.Text = Min.ToString() + Suffix;
        L1.Anchor = ObjectAnchor.Top | ObjectAnchor.Left;
        L2Default.Text = Default.ToString() + Suffix;
        L2Cursor.Text = Value.ToString() + Suffix;
        L3.Text = Max.ToString() + Suffix;
        L3.Anchor = ObjectAnchor.Top | ObjectAnchor.Right;
        L1.Location = new(0, MaxLine.Location.Y - L1.Size.Y - 5.ScaleInt());
        L2Default.Location = new(0, L1.Location.Y);
        L2Default.Color = Color4.Cyan;
        L2Cursor.Location = new(0, L1.Location.Y);
        L3.Location = new(0, L1.Location.Y);
        MaxLine.Anchor = L3.Anchor;
        DefaltLine.Anchor = Cursor.Anchor;
        Controls.Add(progressBar);
        MaxLine.BackgroundColor = Color4.Cyan;
        MinLine.BackgroundColor = MaxLine.BackgroundColor;
        DefaltLine.BackgroundColor = MaxLine.BackgroundColor;
        StopNextHover = true;
        L1.IgnoreHover = true;
        L2Default.IgnoreHover = true;
        L2Cursor.IgnoreHover = true;
        L3.IgnoreHover = true;
        Controls.Add(MinLine);
        Controls.Add(L1);
        Controls.Add(L3);
        Controls.Add(L2Cursor);
        Controls.Add(L2Default);
        Controls.Add(MaxLine);
        Controls.Add(DefaltLine);
        Controls.Add(Cursor); 
        Cursor.ForceDistanceUpdate(this);
        real_x = Cursor.Location.X;
        progressBar.ForceDistanceUpdate(this);
        WindowLoaded += OnWindowLoaded;
    }
    
    private Color4 FillColorL = Globals.DodgerBlue, BackColorL = new(60, 60, 60, 255), CursorColorL = Color4.White, DisabledCursorColorL = new(128, 128, 128, 255), DisabledFillColorL = new(15, 72, 128, 255), DisabledBackColorL = new(40, 40, 40, 255);
    private bool LocalDisabled;

    public bool Disabled
    {
        get => LocalDisabled;
        set
        {
            if (LocalDisabled == value) return;
            LocalDisabled = value;
            AddBlockDraw();
            if (value)
            {
                progressBar.ProgressColor = DisabledFillColorL;
                progressBar.BackgroundColor = DisabledBackColorL;
                RemoveBlockDraw();
                Cursor.BackgroundColor = DisabledCursorColorL;
                return;
            }
            
            progressBar.ProgressColor = FillColorL;
            progressBar.BackgroundColor = BackColorL;
            RemoveBlockDraw();
            Cursor.BackgroundColor = CursorColorL;
        }
    }
    
    public Color4 CursorColor
    {
        get => CursorColorL;
        set
        {
            if (CursorColorL == value) return;
            CursorColorL = value;
            if (Disabled) return;
            Cursor.BackgroundColor = value;
        }
    }
    
    public Color4 DisabledCursorColor
    {
        get => DisabledCursorColorL;
        set
        {
            if (DisabledCursorColorL == value) return;
            DisabledCursorColorL = value;
            if (!Disabled) return;
            Cursor.BackgroundColor = value;
        }
    }

    public Color4 FillColor
    {
        get => FillColorL;
        set
        {
            if (FillColorL == value) return;
            FillColorL = value;
            if (Disabled) return;
            progressBar.ProgressColor = value;
        }
    }
    
    public Color4 DisabledFillColor
    {
        get => DisabledFillColorL;
        set
        {
            if (DisabledFillColorL == value) return;
            DisabledFillColorL = value;
            if (!Disabled) return;
            progressBar.ProgressColor = value;
        }
    }
    
    public Color4 BackColor
    {
        get => BackColorL;
        set
        {
            if (BackColorL == value) return;
            BackColorL = value;
            if (Disabled) return;
            progressBar.BackgroundColor = value;
        }
    }
    
    public Color4 DisabledBackColor
    {
        get => DisabledBackColorL;
        set
        {
            if (DisabledBackColorL == value) return;
            DisabledBackColorL = value;
            if (!Disabled) return;
            progressBar.BackgroundColor = value;
        }
    }

    private Task OnWindowLoaded(RenderObjectCore arg)
    {
        Window!.MouseUp += WindowOnMouseUp;
        Window!.MouseMove += WindowOnMouseMove;
        Cursor.Clicked += CursorOnClicked;
        return Task.CompletedTask;
    }

    private Task ProgressBarOnClicked(RenderObjectCore arg)
    {
        if (Disabled) return Task.CompletedTask;
        Value = progressBar.GetValueFromX((int)Window!.MousePosition.X - progressBar.GetWindowLocation().X) + Min;
        return Task.CompletedTask;
    }

    public void AddPoint(TNumber value)
    {
        if (value < _Min || value > Max) return;
        Rectangle r = new();
        r.Size = MinLine.Size;
        r.Location = new(GetX(value) + space - (r.Size.X / 2), MinLine.Location.Y);
        r.Anchor = ObjectAnchor.PreventWidthChange;
        r.BackgroundColor = MinLine.BackgroundColor;
        r.IgnoreHover = true;
        r.ForceDistanceUpdate(this);
        ExtraPoints.Add(new(value, r));
        Controls.Insert(Controls.IndexOf(MinLine), r);
    }

    private byte BlockEvents = 0;

    public override void SetSize(int w, int h, bool UpdateOpenGLCords = true, bool TrigerSizeEvent = true,
        bool TrigerParentSizeEvent = true, bool TryDraw = true, bool UpdateDistance = false)
    {
        base.SetSize(w, 50.ScaleInt(), UpdateOpenGLCords, TrigerSizeEvent, TrigerParentSizeEvent, TryDraw, UpdateDistance);
        progressBar.SetSize(w - space - space, 8.ScaleInt());
        progressBar.UpdateProgress();
        progressBar.ForceDistanceUpdate(this);
        Min = Min;
        BlockEvents++;
        Value--;
        Value++;
        foreach ((TNumber, Rectangle) Point in ExtraPoints)
        {
            Point.Item2.Location = new(GetX(Point.Item1) + space - (Point.Item2.Size.X / 2), MinLine.Location.Y);
            Point.Item2.ForceDistanceUpdate(this);
        }
        BlockEvents--;
        progressBar.UpdateProgress();
    }

    public bool Draging;

    private int real_x, real_dif;

    private Task CursorOnClicked(RenderObjectCore arg)
    {
        if (Disabled) return Task.CompletedTask;
        Draging = true;
        real_dif = ((int)Window!.MousePosition.X - Cursor.GetWindowLocation().X);
        real_x = Cursor.Location.X + real_dif;
        return Task.CompletedTask;
    }

    private void WindowOnMouseMove(MouseMoveEventArgs obj)
    {
        if (Disabled) return;
        if (!Draging) return;
        real_x += (int)obj.DeltaX;
        TNumber n = progressBar.GetValueFromX(real_x) + Min;
        if (n == Value) return;
        Value = progressBar.GetValueFromX(real_x) + Min;
    }

    private void WindowOnMouseUp(MouseButtonEventArgs obj)
    {
        if (Disabled) return;
        if (!Draging) return;
        Draging = false;
    }
}