using System.Diagnostics;
using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using Luski.GUI.MainScreen.UI.LuskiControls;
using Luski.net.Structures.Main;
using Luski.net.Structures.Public;
using Luski.Shared.PublicServers.V1.Enums;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common.Input;
using OpenTK.Windowing.GraphicsLibraryFramework;
using Label = GraphicsManager.Objects.Label;

namespace Luski.GUI.MainScreen.UI.PublicServers;

public class ChatMessage : UserControl
{
    private SocketMessage Msg { get; }
    private SocketChannel ch { get; }

    public PublicChat pc;

    private RenderObjectCore LastObject;
    public List<RenderObjectCore> MessageObjs = new();
    private LabelBase FirstL;
    public MessageCompressedFlow MCF;

    public readonly double HorPadding = 12.ScaleDouble(),
        VerticalPadding = 5.ScaleDouble();

    private static Dictionary<MainSocketRemoteUser, List<ChatMessage>> Messages = new();

    public static async Task<ChatMessage> MakeChatMessage(PublicChat p, SocketMessage message)
    {
        SocketUser auth = (SocketUser)(await message.GetAuthor(CancellationToken.None));
        ServerProfile prof = await message.GetProfile(CancellationToken.None);
        Role r = (await ((SocketUser)await message.GetAuthor(CancellationToken.None)).GetRoles())[0];
        return new ChatMessage(p, message, await message.GetParent(CancellationToken.None), prof, await prof.MakeRct(auth, new(40.ScaleInt())), r);
    }


    private ChatMessage(PublicChat p, SocketMessage message, SocketChannel chan, ServerProfile Author, RenderObjectCore UserIcon, Role r)
    {
        
        pc = p;
        LuskiLabel label1;
        base.SetSize(723.5.ScaleInt(), 37.ScaleInt());
        ch = chan;
        base.BackgroundColor = new(40, 40, 40, 255);
        Msg = message;
        Anchor = ObjectAnchor.Left | ObjectAnchor.Right;
        
        DateTime time = chan.Epoch.AddMilliseconds(Msg.TimeStamp).ToLocalTime();
        
        string time_str;
        if (Globals.Settings.DayTime)
        {
            if (time.Date == DateTime.Now.ToLocalTime().Date)
            {
                time_str = $"Today at {time:HH:mm}";
            }
            else if (time.Date == DateTime.Now.ToLocalTime().AddDays(-1).Date)
            {
                time_str = $"Yesterday at {time:HH:mm}";
            }
            else
            {
                time_str = $"{time:M/dd/yyyy HH:mm}";
            }
        }
        else
        {
            if (time.Date == DateTime.Now.ToLocalTime().Date)
            {
                time_str = $"Today at {time:h:mm tt}";
            }
            else if (time.Date == DateTime.Now.ToLocalTime().AddDays(-1).Date)
            {
                time_str = $"Yesterday at {time:h:mm tt}";
            }
            else
            {
                time_str = $"{time:M/dd/yyyy h:mm tt}";
            }
        }
       
        UserIcon.Location = new(10.ScaleInt(), 2.ScaleInt());
        Controls.Add(UserIcon);
        string name = Author.DisplayName;
        if (r.ColorType == ColorType.Full)
        {
            name = $"[color=\"{r.Colors[0].ToDatabaseStr()}\"]{name}[/color]";
        }
        else
        {
            name = $"[gradient colors=\"{r.Colors.ToDB()}\"]{name}[/gradient]";
        }
        Controls.Add(label1 = new (Globals.DefaultFont) { Text = name });
        label1.Location = new(
            54.ScaleInt(),
            UserIcon.Location.Y);
        Label label2;
        LastObject = label1;
        FirstL = label1;
        Controls.Add(label2 = new(Globals.TopTimeFont) { Location = new(label1.Location.X + label1.Size.X + 8.ScaleInt(), (int)(label1.Location.Y + label1.Font.PixelHeight - Globals.TopTimeFont.PixelHeight)), Text = time_str});
        
        MCF = new()
        {
            Location = new(LastObject.Location.X, (int)(UserIcon.Location.Y + UserIcon.Size.Y - Globals.MessageFont.PixelHeight)),
            BackgroundColor = new(0,0,0,0),
            Size = new(base.Size.X - LastObject.Location.X - 5.ScaleInt(), 0),
            Anchor = ObjectAnchor.Top | ObjectAnchor.Left | ObjectAnchor.Right,
        };
        MCF.SizeUpdateNotIgnored += MCFOnSizeUpdateNotIgnored;
        MCF.ForceDistanceUpdate(this);
        Controls.Add(MCF);
        
        if (!string.IsNullOrWhiteSpace(Msg.Context))
        {
            LuskiLabel l;
            MCF.Controls.Add(l = new(Globals.MessageFont) { Location = new(LastObject.Location.X, (int)(UserIcon.Location.Y + UserIcon.Size.Y - Globals.MessageFont.PixelHeight)), Text = message.Context});
            LastObject = l;
            LuskiContextMenu lcm = new();
            Label llllll = lcm.AddLabel("Copy Text");
            unsafe
            {
                llllll.Clicked += (o) =>
                {
                    GLFW.SetClipboardString(Globals.ms.WindowPtr, message.Context);
                    return Task.CompletedTask;
                };
            }
            l.ContextMenu = lcm;
            MessageObjs.Add(l);
        }
        Globals.Settings.DayTimeChanged += () => 
        { 
            if (Globals.Settings.DayTime)
            {
                if (time.Date == DateTime.Now.ToLocalTime().Date)
                {
                    time_str = $"Today at {time:HH:mm}";
                }
                else if (time.Date == DateTime.Now.ToLocalTime().AddDays(-1).Date)
                {
                    time_str = $"Yesterday at {time:HH:mm}";
                }
                else
                {
                    time_str = $"{time:M/dd/yyyy HH:mm}";
                }
            }
            else
            {
                if (time.Date == DateTime.Now.ToLocalTime().Date)
                {
                    time_str = $"Today at {time.ToShortTimeString().Replace('\u202f', ' ')}";
                }
                else if (time.Date == DateTime.Now.ToLocalTime().AddDays(-1).Date)
                {
                    time_str = $"Yesterday at {time.ToShortTimeString().Replace('\u202f', ' ')}";
                }
                else
                {
                    time_str = $"{time:M/dd/yyyy h:mm tt}";
                }
            }

            label2.Text = time_str;
            return Task.CompletedTask;
        };
        
        if (Msg.Files.Count > 0)
        {
            CompressedFlow cf = new()
            {
                Size = new(base.Size.X - FirstL.Location.X, 0),
                Anchor = ObjectAnchor.Left | ObjectAnchor.Right | ObjectAnchor.Top,
                BackgroundColor = new(0,0,0,50)
            };
            for (int i = 0; i < Msg.Files.Count; i++)
            {
                var cem = ContentEmbed.GetEmbed(this, Msg.Files[i], Msg.ChannelID);
                cem.Wait();
                cf.Controls.Add(cem.Result);
            }
            MCF.Controls.Add(cf);
        }

        base.Size = new(base.Size.X , MCF.Size.Y + MCF.Location.Y);
        base.BackgroundColor = new(0,0,0,50);
    }

    private Task MCFOnSizeUpdateNotIgnored(RenderObjectCore arg)
    {
        base.Size = new(base.Size.X , MCF.Size.Y + MCF.Location.Y);
        return Task.CompletedTask;
    }


    public async Task AddMessage(SocketMessage msg)
    {
        AddBlockDraw();
        LuskiLabel newLabel;
        if (!string.IsNullOrWhiteSpace(msg.Context))
        {
            newLabel = new(Globals.MessageFont)
            {
                Text = msg.Context,
                Tag = msg
            };
            if (LastObject is Label l)
            {
                newLabel.Location = new(FirstL.Location.X, (int)(l.Location.Y + l.Size.Y + VerticalPadding));
            }
            else
            {
                newLabel.Location = new(FirstL.Location.X, Size.Y);
            }
            bool result = Uri.TryCreate(newLabel.Text, UriKind.Absolute, out Uri? uriResult) 
                          && (uriResult.Scheme == Uri.UriSchemeHttp || uriResult.Scheme == Uri.UriSchemeHttps);
            if (result)
            {
                newLabel.HoverMouse = MouseCursor.Hand;
                newLabel.DefaultColor = Color4.Aqua;
                newLabel.Clicked += NewLabelOnClicked;
            }
            
            newLabel.MouseEnter += NewLabel_MouseEnter;
            newLabel.MouseLeave += NewLabel_MouseLeave;
            MCF.Controls.Add(newLabel);
            MessageObjs.Add(newLabel);
            LastObject = newLabel;
            LuskiContextMenu lcm = new();
            Label llllll = lcm.AddLabel("Copy Text");
            unsafe
            {
                llllll.Clicked += (o) =>
                {
                    GLFW.SetClipboardString(Globals.ms.WindowPtr, msg.Context);
                    return Task.CompletedTask;
                };
            }
            newLabel.ContextMenu = lcm;
        }
        
        if (msg.Files.Count > 0)
        {
            CompressedFlow cf = new()
            {
                Size = new(base.Size.X - FirstL.Location.X, 0),
                Anchor = ObjectAnchor.Left | ObjectAnchor.Right | ObjectAnchor.Top,
                BackgroundColor = new(0,0,0,0)
            };
            for (int i = 0; i < msg.Files.Count; i++)
            {
                RenderObjectCore cem = await ContentEmbed.GetEmbed(this, msg.Files[i], msg.ChannelID);
                cf.Controls.Add(cem);
            }
            MCF.Controls.Add(cf);
        }
        
        Size = new(Size.X , MCF.Size.Y + MCF.Location.Y);
        
        RemoveBlockDraw();
        TryDraw();
    }

    private Task NewLabelOnClicked(RenderObjectCore arg)
    {
        try
        {
            Label m = (arg as Label)!;
            if (OperatingSystem.IsWindows())
                Process.Start(m.Text);
            else if (OperatingSystem.IsLinux())
            {
                if (m.Tag is string s) Process.Start("xdg-open", s);
                else Process.Start("xdg-open", m.Text);
            }
        }
        catch (Exception e)
        {
            Console.WriteLine(e);
        }
        return Task.CompletedTask;
    }

    readonly List<Label> Labels = new();
    private async Task NewLabel_MouseLeave(RenderObjectCore sender)
    {
        if (sender is not Label label) return;
        if (label.Tag is not SocketMessage Message) return;
        DateTime time = (await Message.GetParent(CancellationToken.None)).Epoch.AddMilliseconds(Message.TimeStamp).ToLocalTime();
        string b; 
        if (!Globals.Settings.DayTime) b = time.ToString("h:mm tt");
        else b = time.ToString("HH:mm");
        Globals.Settings.DayTimeChanged += () => 
        { 
            if (!Globals.Settings.DayTime) b = time.ToString("h:mm tt");
            else b = time.ToString("HH:mm"); 
            return Task.CompletedTask;
            
        };
        Label[] l = Labels.Where(s => s.Text == b).ToArray();
        if (l.Any())
        {
            Controls.Remove(l.First());
            Labels.Remove(l.First());
        }
        Window!.TryDraw();
    }
    
    private async Task NewLabel_MouseEnter(RenderObjectCore sender)
    {
        if (sender is not Label label) return;
        if (label.Tag is not SocketMessage Message) return;
        DateTime time = (await Message.GetParent(CancellationToken.None)).Epoch.AddMilliseconds(Message.TimeStamp).ToLocalTime();
        Label m;
        if (!Globals.Settings.DayTime) m = new( Globals.SmallTimeFont)
        {
            Text = time.ToString("h:mm tt"),
        };
        else m = new( Globals.SmallTimeFont)
        {
            Text = time.ToString("HH:mm"),
        };
        Globals.Settings.DayTimeChanged += () => 
        { 
            if (!Globals.Settings.DayTime) m.Text = time.ToString("h:mm tt");
            else m.Text = time.ToString("HH:mm");
            return Task.CompletedTask;
        };
        m.Location = new(
            label.Location.X - m.Size.X - 5.ScaleInt(),
            (int)(label.Location.Y + label.Font.PixelHeight - Globals.SmallTimeFont.PixelHeight));
        
        Controls.Add(m);
        Labels.Add(m);
        Window!.TryDraw();
    }
}